;*************************************************************************************
;*                                     DRAM Audio Recorder                           *
;*************************************************************************************
;*                                                                                   *
;*   Einfaches Testprogramm, das Audiodaten aufzeichnet und wieder abspielt.	     *
;*   im Hauptptogramm.                                                               *
;*                                                                                   *
;*   Hier wird ein 1M x 16 DRAM verwendet, da dieser relativ weit verbreitet ist.    *
;*   D0-7 sind mit D8-D15 verbunden um Pins zu sparen und sind an PORT D             *
;*   angeschlossen. Ebenso A0-A7. A8 und A9 bekommen einen eigenen Pin an PortC      *
;*   WE, OE, UCAS, LCAS und RAS an PORT B. 2MByte an 15 Pins ist doch ganz OK.       *
;*   Mit einer Samplerate von 9,6kHz bekommt man so immerhin 3,5Minuten Audio in den *
;*   Speicher. Die Qualitt wird hauptschlich von der niedrigen PWM Frequenz von    *                                                                               *
;*   31kHz negativ beeinflusst.                                                      *
;*                                                                                   *
;*************************************************************************************

.include "m8def.inc"	;@ 8MHz

.def eins=r1
.def voll=r2
.def null=r3
.def buff1=r4
.def buff2=r5
.def fz=r6

.def temp=r16
.def AdresseLow=r17
.def AdresseMid=r18
.def AdresseHigh=r19
.def Byte=r20
.def Refresh=r21
.def Bits=r22

;Port B
.equ RecLED=6
.equ PlayLED=7
.equ OE=5
.equ UCAS=4
.equ LCAS=3
.equ WE=2
.equ RAS=0

;PortC
.equ Play=1		;Play Button
.equ Stop=2		;Stop Button
.equ Record=3	;Record Button
.equ A8=4
.equ A9=5


;Bits
.equ Refr=0
.equ MPlay=1
.equ MRecord=2
.equ Nibble=3

.org 0000
rjmp reset

.org OVF0addr	        	;Timer 0
reti

.org ADCCaddr
rjmp adcready


Reset:
ldi temp, 255		        ;PORTB Steuerleitungen = Ausgang
out ddrb, temp

ldi temp, 32+16
out ddrc, temp

ldi temp, 2+4+8
out PORTC, temp

ldi temp, 15
mov fz, temp


cbi PORTC, A8
cbi PORTC, A9	            ;Initialisierung der Steuerleitungen
sbi PORTB, RAS
sbi PORTB, UCAS
sbi PORTB, LCAS
sbi PORTB, OE
sbi PORTB, WE

ser temp		            ;PORTD Adress/Daten = Ausgang (und Eingang)
out ddrd, temp

ldi		r16, low(RAMEND)
out		SPL, r16	    	; setup stack pointer

ldi		r16, high(RAMEND)
out		SPH, r16	    	; setup stack pointer


clr null
ldi temp, 1
mov eins, temp
ser temp
mov voll, temp

clr Bits

ldi temp, 224
out admux, temp

ldi temp, 6+64+32+8		    ;ADC init: 2,56V VRef, ADC0, fClk/64
out ADCSR, temp

ldi temp, 128+64+1	    	;Fast PWM als DAC
out tccr1a, temp

ldi temp, 8
out tccr1b, temp

ldi temp, 5			    	;Refresh Timer erzeugt alle 22ms
out TCCR0, temp		    	;einen Interrupt und startet den
					    	;Burst Refresh.
out TCNT0, null		    	;Da 1024 Zeilen refreshed werden,
					    	;bentigt ein Refresh 4x 256 Takte,
ldi temp, 1			    	;also 4x22ms=88ms, etwas ber den 64ms
out TIMSK, temp			    ;aus dem Datenblatt, funktioniert aber und spart CPU Zeit
					        ;Insgesamt sollte man aber unter 100ms bleiben.

clr AdresseLow		    	;24bit Adresszhler (21bit bentigt fr 2MByte)
clr AdresseMid
clr AdresseHigh

clr Refresh				    ;Refreshzhler

ldi Byte, 255
ramtst:					        ;Memory Test
	rcall WriteLowByte
	inc AdresseLow
	dec Byte
	dec Byte
	cpi AdresseLow, 128
	brne ramtst

ldi AdresseLow, 0
ldi temp, 255
ramtst2:
	rcall ReadLowByte
	cp Byte, temp
	brne Ende
	inc AdresseLow
	dec temp
	dec temp
	cpi AdresseLow, 128
	brne ramtst2


ldi AdresseLow, 0

sei

rjmp Refresh

Ende:
	sbi portc, RecLED	    	;Memory Test failed !
Ende2:
rjmp Ende2



Refresh:
	sbr Bits, (1<<Refr)	    	;Refresh Bit setzen: Daran erkennt ein Interrupt ob gerade ein Refresh aktiv ist.
	cbi PORTB, LCAS		        ;Init CBR Refresh
Main:
 	cbi PORTB, RAS		        ;toogle RAS
	add Refresh, eins       
 	sbi PORTB, RAS
	brcc Main

 	sbi PORTB, LCAS	        	;End CBR Refresh
	ldi temp, 128	        	;Set Idle Mode
	out mcucr, temp
	nop
	sleep				
	nop

	sbic pinc, Play		    	;Play taste gedrckt
	rjmp skips1
	sbrc Bits, MRecord	    	;Wenn Aufnahme, dann keine Wiedergabe mglich
	rjmp skips1
	sbi adcsr, ADEN		    	;Enable ADC
	ldi temp, 8+1
	out tccr1b, temp		    ;Enable PWM
	sbi portb, PlayLED
	sbrc Bits, MPlay
	rjmp skips1

	sbr Bits, (1<<MPlay)
	cbr Bits, (1<<Nibble)

skips1:	
	sbic pinc, Record
	rjmp skips2
	sbrc Bits, MPlay	    	;Wenn Wiederabe, dann keine Aufnahme mglich
	rjmp skips1
	sbi adcsr, ADEN		    	;Enable ADC
	ldi temp, 8+1
	out tccr1b, temp	    	;Enable PWM
	sbi portb, RecLED
	sbrc Bits, MRecord
	rjmp skips2
	sbr Bits, (1<<MRecord)
	cbr Bits, (1<<Nibble)
skips2:
	sbic pinc, Stop
	rjmp skips
	cbr Bits, (1<<MRecord)|(1<<MPlay)
	cbi adcsr, ADEN		    	;Disable ADC
	ldi temp, (1<<Nibble)
	out tccr1b, temp	    	;Disable PWM
	clr AdresseLow
	clr AdresseMid
	clr AdresseHigh	
	cbi portb, PlayLED
	cbi portb, RecLED
skips:
rjmp Refresh


ReadLowByte:
	out PORTD, AdresseLow		;Low Adressbyte ausgeben
	cbi PORTC, A8				;A8 Ausgeben, ist beim AVR etwas kompliziert, da kein direkter Bit -> IO Transfer mglich ist
	sbrc AdresseHigh, 0
	sbi PORTC, A8	 

	cbi PORTC, A9				;A9 Ausgeben, ist beim AVR etwas kompliziert, da kein direkter Bit -> IO Transfer mglich ist
	sbrc AdresseHigh, 1
	sbi PORTC, A9	 
	
	nop
	cbi PORTB, RAS				;Zeilen Adresse im DRAM Speichern

	out PORTD, AdresseMid		;Mid Adressbyte ausgeben
	cbi PORTC, A8				;A8 Ausgeben, ist beim AVR etwas kompliziert, da kein direkter Bit -> IO Transfer mglich ist
	sbrc AdresseHigh, 2
	sbi PORTC, A8	 

	cbi PORTC, A9				;A9 Ausgeben, ist beim AVR etwas kompliziert, da kein direkter Bit -> IO Transfer mglich ist
	sbrc AdresseHigh, 3
	sbi PORTC, A9	 
	
	nop
	cbi PORTB, LCAS				;Spalten Adresse im DRAM Speichern

	out ddrd, null				;IO auf Eingang schalten
	
	cbi PORTB, OE				;OE aktivieren

	nop
	nop
	in Byte, pind				;Byte lesen

	sbi PORTB, OE				;OE deaktivieren

	out ddrd, voll				;IO auf Ausgang schalten

	sbi PORTB, LCAS				;CAS deaktivieren

	sbi PORTB, RAS				;RAS deaktivieren
ret

ReadHighByte:
	out PORTD, AdresseLow		;Low Adressbyte ausgeben
	cbi PORTC, A8				;A8 Ausgeben, ist beim AVR etwas kompliziert, da kein direkter Bit -> IO Transfer mglich ist
	sbrc AdresseHigh, 0
	sbi PORTC, A8	 

	cbi PORTC, A9				;A9 Ausgeben, ist beim AVR etwas kompliziert, da kein direkter Bit -> IO Transfer mglich ist
	sbrc AdresseHigh, 1
	sbi PORTC, A9	 
	
	nop
	cbi PORTB, RAS				;Zeilen Adresse im DRAM Speichern

	out PORTD, AdresseMid		;Mid Adressbyte ausgeben
	cbi PORTC, A8				;A8 Ausgeben, ist beim AVR etwas kompliziert, da kein direkter Bit -> IO Transfer mglich ist
	sbrc AdresseHigh, 2
	sbi PORTC, A8	 

	cbi PORTC, A9				;A9 Ausgeben, ist beim AVR etwas kompliziert, da kein direkter Bit -> IO Transfer mglich ist
	sbrc AdresseHigh, 3
	sbi PORTC, A9		 

	nop
	cbi PORTB, UCAS				;Spalten Adresse im DRAM Speichern

	out ddrd, null				;IO auf Eingang schalten
	
	cbi PORTB, OE				;OE aktivieren

	nop
	nop
	in Byte, pind				;Byte lesen

	sbi PORTB, OE				;OE deaktivieren

	out ddrd, voll				;IO auf Ausgang schalten

	sbi PORTB, UCAS				;CAS deaktivieren

	sbi PORTB, RAS				;RAS deaktivieren
ret


WriteLowByte:
	out PORTD, AdresseLow		;Low Adressbyte ausgeben
	cbi PORTC, A8				;A8 Ausgeben, ist beim AVR etwas kompliziert, da kein direkter Bit -> IO Transfer mglich ist
	sbrc AdresseHigh, 0
	sbi PORTC, A8	 

	cbi PORTC, A9				;A9 Ausgeben, ist beim AVR etwas kompliziert, da kein direkter Bit -> IO Transfer mglich ist
	sbrc AdresseHigh, 1
	sbi PORTC, A9	 
	
	nop
	cbi PORTB, RAS				;Zeilen Adresse im DRAM Speichern

	out PORTD, AdresseMid		;Mid Adressbyte ausgeben
	cbi PORTC, A8				;A8 Ausgeben, ist beim AVR etwas kompliziert, da kein direkter Bit -> IO Transfer mglich ist
	sbrc AdresseHigh, 2
	sbi PORTC, A8	 

	cbi PORTC, A9				;A9 Ausgeben, ist beim AVR etwas kompliziert, da kein direkter Bit -> IO Transfer mglich ist
	sbrc AdresseHigh, 3
	sbi PORTC, A9	
	 
	nop
	cbi PORTB, LCAS				;Spalten Adresse im DRAM Speichern

	out PORTD, Byte				;Byte ausgeben

	nop
	cbi PORTB, WE				;Eearly Write aktivieren

	nop
	sbi PORTB, WE				;Write deaktivieren

	sbi PORTB, LCAS				;CAS deaktivieren


	sbi PORTB, RAS				;RAS deaktivieren
ret

WriteHighByte:
	out PORTD, AdresseLow		;Low Adressbyte ausgeben
	cbi PORTC, A8				;A8 Ausgeben, ist beim AVR etwas kompliziert, da kein direkter Bit -> IO Transfer mglich ist
	sbrc AdresseHigh, 0
	sbi PORTC, A8	 

	cbi PORTC, A9				;A9 Ausgeben, ist beim AVR etwas kompliziert, da kein direkter Bit -> IO Transfer mglich ist
	sbrc AdresseHigh, 1
	sbi PORTC, A9	 
	
	nop
	cbi PORTB, RAS				;Zeilen Adresse im DRAM Speichern

	out PORTD, AdresseMid		;Mid Adressbyte ausgeben
	cbi PORTC, A8				;A8 Ausgeben, ist beim AVR etwas kompliziert, da kein direkter Bit -> IO Transfer mglich ist
	sbrc AdresseHigh, 2
	sbi PORTC, A8	 

	cbi PORTC, A9				;A9 Ausgeben, ist beim AVR etwas kompliziert, da kein direkter Bit -> IO Transfer mglich ist
	sbrc AdresseHigh, 3
	sbi PORTC, A9	

	nop
	cbi PORTB, UCAS				;Spalten Adresse im DRAM Speichern

	out PORTD, Byte				;Byte ausgeben

	nop
	cbi PORTB, WE				;Eearly Write aktivieren

	nop
	sbi PORTB, WE				;Write deaktivieren

	sbi PORTB, UCAS				;CAS deaktivieren


	sbi PORTB, RAS				;RAS deaktivieren
ret



adcready:
	push temp
	sbi PORTB, RAS				;Disable Refresh
	sbi PORTB, LCAS

	sbrs Bits, MRecord
	rjmp skipr
	in Byte, adch				;neuen Wert einlesen

	sbrs AdresseHigh, 4			;Eigentlich wre ein Read-modify-write
	rcall WriteLowByte			;hier besser, aber das ganze ist ja nur ein Test
	sbrc AdresseHigh, 4
	rcall WriteHighByte

	add AdresseLow,eins			;24 bit Adresszhler hochzhlen
	adc AdresseMid,null
	adc AdresseHigh,null
	sbrs AdresseHigh,5			;Speicher voll ?
	rjmp intexit

	cbr Bits, (1<<MRecord)|(1<<MPlay)	;Speicher voll !
	cbi adcsr, ADEN				;Disable ADC
	ldi temp, 8
	out tccr1b, temp			;Disable PWM	
	clr AdresseLow
	clr AdresseMid
	clr AdresseHigh	
	cbi portb, PlayLED
	cbi portb, RecLED
rjmp intexit

skipr:
	sbrs Bits, MPlay
	rjmp intexit

skipln2:
	sbr Bits, (1<<Nibble)		;Lownibble
	sbrs AdresseHigh, 4			;Da 16bit Speicher, 
	rcall ReadLowByte			;entscheidet das MSBit
	sbrc AdresseHigh, 4			;ob Low oder Highbyte
	rcall ReadHighByte

	out ocr1al, Byte			;gespeicherten Wert ausgeben

	add AdresseLow,eins			;24 bit Adresszhler hochzhlen
	adc AdresseMid,null
	adc AdresseHigh,null
	sbrs AdresseHigh,5			;Speicher voll ?
	rjmp intexit
	cbr Bits, (1<<MRecord)|(1<<MPlay)
	cbi adcsr, ADEN				;Disable ADC
	ldi temp, 8
	out tccr1b, temp			;Disable PWM	
	clr AdresseLow
	clr AdresseMid
	clr AdresseHigh	
	cbi portb, PlayLED
	cbi portb, RecLED

intexit:
	sbrc Bits, Refr				;War ein Refresh aktiv, als der Interrupt kam ?
	cbi PORTB, LCAS				;Wenn ja, dann alten Zustand widerherstellen
	pop temp
reti


